@@ -13,9 +13,12 @@
 //---------------------------------------------------------------------------
 #include "Checker.h"
 
+#include <functional>
+#include <algorithm>
 #include <cstdlib>
 #include <cstring>
 #include <sstream>
+
 //TODO: mmt
 //TODO: reference_file
 
@@ -37,7 +40,7 @@ std::string xml_encode(const std::string& data)
             case '&': ss << "&amp;"; break;
             case '<': ss << "&lt;"; break;
             case '>': ss << "&gt;"; break;
-            case '\t': ss << "&#x9;";
+            case '\t': ss << "&#x9;"; break;
             case '\n': ss << "&#xA;"; break;
             case '\r':
                 ss << "&#xA;";
@@ -50,53 +53,6 @@ std::string xml_encode(const std::string& data)
     return ss.str();
 }
 
-//---------------------------------------------------------------------------
-bool compare(const std::string& value, const std::string& reference, const std::string& operand)
-{
-    char* val_end=NULL;
-    double val = strtod(value.c_str(), &val_end);
-    char* ref_end=NULL;
-    double ref = strtod(reference.c_str(), &ref_end);
-
-    if (operand == "<")
-    {
-        if (!strlen(val_end) && !strlen(ref_end))
-            return val < ref;
-        else
-            return strcmp(value.c_str(), reference.c_str()) < 0;
-    }
-    else if (operand == "<=")
-    {
-        if (!strlen(val_end) && !strlen(ref_end))
-            return val <= ref;
-        else
-            return strcmp(value.c_str(), reference.c_str()) <= 0;
-    }
-    else if (operand == "=")
-    {
-        if (!strlen(val_end) && !strlen(ref_end))
-            return val == ref;
-        else
-            return strcmp(value.c_str(), reference.c_str()) == 0;
-    }
-    else if (operand == ">=")
-    {
-        if (!strlen(val_end) && !strlen(ref_end))
-            return val >= ref;
-        else
-            return strcmp(value.c_str(), reference.c_str()) >= 0;
-    }
-    else if (operand == ">")
-    {
-        if (!strlen(val_end) && !strlen(ref_end))
-            return val > ref;
-        else
-            return strcmp(value.c_str(), reference.c_str()) > 0;
-    }
-
-    return false;
-}
-
 //---------------------------------------------------------------------------
 std::string indent(size_t level)
 {
@@ -140,7 +96,7 @@ namespace MediaConch {
 //---------------------------------------------------------------------------
 
 //---------------------------------------------------------------------------
-PolicyChecker::PolicyElement::PolicyElement() : resolved(false), pass_count(0), fail_count(0)
+PolicyChecker::PolicyElement::PolicyElement() : resolved(false), pass_count(0), info_count(0), warn_count(0), fail_count(0)
 {
 }
 
@@ -154,27 +110,84 @@ PolicyChecker::PolicyElement::~PolicyElement()
     }
 }
 
+//---------------------------------------------------------------------------
+PolicyChecker::Element::Level PolicyChecker::PolicyElement::error_level()
+{
+    if (level=="info")
+        return LEVEL_INFO;
+    else if (level=="warn")
+        return LEVEL_WARN;
+    else if (level=="fail")
+        return LEVEL_FAIL;
+
+    return LEVEL_NONE;
+}
+
 //---------------------------------------------------------------------------
 void PolicyChecker::PolicyElement::resolve()
 {
     for (size_t pos=0; pos<children.size(); pos++)
     {
-        if (children[pos]->result())
-             pass_count++;
-         else
-             fail_count++;
+        switch (children[pos]->result())
+        {
+        case RESULT_PASS:
+            pass_count++; break;
+        case RESULT_INFO:
+            info_count++; break;
+        case RESULT_WARN:
+            warn_count++; break;
+        case RESULT_FAIL:
+            fail_count++; break;
+        default:
+            break;
+        }
     }
 
     resolved=true;
 }
 
 //---------------------------------------------------------------------------
-bool PolicyChecker::PolicyElement::result()
+PolicyChecker::Element::Result PolicyChecker::PolicyElement::result()
 {
     if(!resolved)
         resolve();
 
-    return (type=="or" && pass_count) || (type=="and" && !fail_count);
+    Result to_return=RESULT_PASS;
+
+    if (type=="and")
+    {
+        if (fail_count)
+            to_return=RESULT_FAIL;
+        else if (warn_count)
+            to_return=RESULT_WARN;
+        else if (info_count)
+            to_return=RESULT_INFO;
+        else
+            to_return=RESULT_PASS;
+    }
+    else if (type=="or")
+    {
+        if (fail_count && !warn_count && !info_count && !pass_count)
+            to_return=RESULT_FAIL;
+        else if (warn_count && !info_count && !pass_count)
+            to_return=RESULT_WARN;
+        else if (info_count && !pass_count)
+            to_return=RESULT_INFO;
+        else
+            to_return=RESULT_PASS;
+    }
+
+    if (error_level()>LEVEL_NONE)
+    {
+        if (to_return>RESULT_INFO && error_level()==LEVEL_INFO)
+            to_return=RESULT_INFO;
+        else if (to_return>RESULT_WARN && error_level()==LEVEL_WARN)
+            to_return=RESULT_WARN;
+    }
+
+
+
+    return to_return;
 }
 
 //---------------------------------------------------------------------------
@@ -183,10 +196,26 @@ std::string PolicyChecker::PolicyElement::to_string(size_t level, bool verbose)
     if(!resolved)
         resolve();
 
+    std::string outcome_str;
+    switch (result())
+    {
+    case RESULT_FAIL:
+        outcome_str="fail"; break;
+    case RESULT_WARN:
+        outcome_str="warn"; break;
+    case RESULT_INFO:
+        outcome_str="info"; break;
+    case RESULT_PASS:
+        outcome_str="pass"; break;
+    }
+
     std::stringstream ss;
 
-    ss << indent(level) << "<policy name=\"" << xml_encode(name) << "\" type=\"" << xml_encode(type) << "\" rules_run=\"" << children.size() << "\" pass_count=\""
-                        << pass_count << "\" fail_count=\"" << fail_count << "\" outcome=\"" << (result() ? "pass" : "fail") << "\">" << std::endl;
+    ss << indent(level) << "<policy name=\"" << xml_encode(name) << "\" type=\"" << xml_encode(type);
+    if (!this->level.empty())
+        ss << "\" level=\"" << xml_encode(this->level);
+    ss << "\" rules_run=\"" << children.size() << "\" pass_count=\"" << pass_count << "\" info_count=\"" << info_count << "\" warn_count=\"" << warn_count
+       << "\" fail_count=\"" << fail_count << "\" outcome=\"" << outcome_str << "\">" << std::endl;
     level++;
     if (!description.empty())
         ss << indent(level) << "<description>" << description << "</description>" << std::endl;
@@ -217,11 +246,84 @@ PolicyChecker::RuleElement::~RuleElement()
 //---------------------------------------------------------------------------
 void PolicyChecker::RuleElement::reset()
 {
-    pass=false;
     resolved=false;
+    pass=false;
     values.clear();
 }
 
+//---------------------------------------------------------------------------
+bool  PolicyChecker::RuleElement::compare(const std::string& value)
+{
+    bool to_return = false;
+
+    char* val_end=NULL;
+    double val = strtod(value.c_str(), &val_end);
+    char* ref_end=NULL;
+    double ref = strtod(requested.c_str(), &ref_end);
+
+    if (operand=="starts with")
+    {
+        to_return = value.rfind(requested, 0) == 0;
+    }
+    else if (operand=="must not start with")
+    {
+        to_return = value.rfind(requested, 0) != 0;
+    }
+    else if (operand == "<")
+    {
+        if (!strlen(val_end) && !strlen(ref_end))
+            to_return = val < ref;
+        else
+            to_return = strcmp(value.c_str(), requested.c_str()) < 0;
+    }
+    else if (operand == "<=")
+    {
+        if (!strlen(val_end) && !strlen(ref_end))
+            to_return = val <= ref;
+        else
+            to_return = strcmp(value.c_str(), requested.c_str()) <= 0;
+    }
+    else if (operand == "=")
+    {
+        if (!strlen(val_end) && !strlen(ref_end))
+            to_return = val == ref;
+        else
+            to_return = strcmp(value.c_str(), requested.c_str()) == 0;
+    }
+    else if (operand == ">=")
+    {
+        if (!strlen(val_end) && !strlen(ref_end))
+            to_return = val >= ref;
+        else
+            to_return = strcmp(value.c_str(), requested.c_str()) >= 0;
+    }
+    else if (operand == ">")
+    {
+        if (!strlen(val_end) && !strlen(ref_end))
+            to_return = val > ref;
+        else
+            to_return = strcmp(value.c_str(), requested.c_str()) > 0;
+    }
+
+    if (!to_return)
+        failing_values.push_back(value);
+
+    return to_return;
+}
+
+//---------------------------------------------------------------------------
+PolicyChecker::Element::Level PolicyChecker::RuleElement::error_level()
+{
+    if (level=="info")
+        return LEVEL_INFO;
+    else if (level=="warn")
+        return LEVEL_WARN;
+    else if (level=="fail")
+        return LEVEL_FAIL;
+
+    return LEVEL_NONE;
+}
+
 //---------------------------------------------------------------------------
 void PolicyChecker::RuleElement::resolve()
 {
@@ -233,48 +335,71 @@ void PolicyChecker::RuleElement::resolve()
     {
             pass=values.empty();
     }
-    else if (operand=="starts with")
+    else if (operand=="starts with" || operand=="must not start with" || operand=="<" || operand=="<=" || operand=="=" || operand==">=" || operand==">")
     {
-            for (std::vector<std::string>::iterator it=values.begin(); it!=values.end(); it++)
-                pass|=(it->rfind(requested, 0)==0);
-    }
-    else if (operand=="must no starts with")
-    {
-            for (std::vector<std::string>::iterator it=values.begin(); it!=values.end(); it++)
-                pass|=(it->rfind(requested, 0)!=0);
-    }
-    else if (operand=="<" || operand=="<=" || operand=="=" || operand==">=" || operand==">")
-    {
-            for (std::vector<std::string>::iterator it=values.begin(); it!=values.end(); it++)
-                 pass|=compare(*it, requested, operand);
+       if (values.empty())
+          pass = operand=="must not start with" ? true : false;
+       else if (occurrence=="all")
+           pass = std::all_of(values.begin(), values.end(), std::bind(&PolicyChecker::RuleElement::compare, this, std::placeholders::_1));
+       else
+           pass = std::any_of(values.begin(), values.end(), std::bind(&PolicyChecker::RuleElement::compare, this, std::placeholders::_1));
     }
 
     resolved=true;
 }
 
 //---------------------------------------------------------------------------
-bool PolicyChecker::RuleElement::result()
+PolicyChecker::Element::Result PolicyChecker::RuleElement::result()
 {
     if(!resolved)
         resolve();
 
-    return pass;
+    if (!pass)
+    {
+        if (error_level()==LEVEL_INFO)
+            return RESULT_INFO;
+        else if (error_level()==LEVEL_WARN)
+            return RESULT_WARN;
+        else
+            return RESULT_FAIL;
+    }
+    return RESULT_PASS;
 }
 
 //---------------------------------------------------------------------------
 std::string PolicyChecker::RuleElement::to_string(size_t level, bool verbose)
 {
     std::stringstream ss;
-    bool outcome = result();
+
+    std::string outcome_str;
+    switch (result())
+    {
+    case RESULT_FAIL:
+        outcome_str="fail"; break;
+    case RESULT_WARN:
+        outcome_str="warn"; break;
+    case RESULT_INFO:
+        outcome_str="info"; break;
+    case RESULT_PASS:
+        outcome_str="pass"; break;
+    }
+
+    std::string value_str;
+    if (result()>RESULT_PASS)
+        value_str=failing_values.size() ? failing_values.front() : std::string();
+    else if (verbose)
+        value_str=values.size() ? values.front() : std::string();
 
     ss << indent(level) << "<rule name=\"" << xml_encode(name) << "\"";
     if (!scope.empty())
         ss << " scope=\"" << xml_encode(scope) << "\"";
     ss << " value=\"" << xml_encode(field) << "\" tracktype=\"" << xml_encode(tracktype) << "\" occurrence=\"" << xml_encode(occurrence)
        << "\" operator=\"" << xml_encode(operand) << "\" xpath=\"" << xpath << "\"";
-    if (!outcome || verbose)
-        ss << " requested=\"" << xml_encode(requested) << "\" actual=\"" << (values.size() ? xml_encode(values.front()) : std::string()) << "\"";
-    ss << " outcome=\"" << (outcome ? "pass" : "fail") << "\"/>" << std::endl;
+    if (!this->level.empty())
+        ss << " level=\"" << xml_encode(this->level) << "\"";
+    if (result()>RESULT_PASS || verbose)
+        ss << " requested=\"" << xml_encode(requested) << "\" actual=\"" << xml_encode(value_str) << "\"";
+    ss << " outcome=\"" << outcome_str << "\"/>" << std::endl;
 
     return ss.str();
 }
@@ -284,7 +409,7 @@ std::string PolicyChecker::RuleElement::to_string(size_t level, bool verbose)
 //---------------------------------------------------------------------------
 
 //---------------------------------------------------------------------------
-PolicyChecker::PolicyChecker()
+PolicyChecker::PolicyChecker() : supported(true), full(false)
 {
 }
 
@@ -298,19 +423,17 @@ PolicyChecker::~PolicyChecker()
     }
 }
 
+//---------------------------------------------------------------------------
+bool PolicyChecker::is_policy_supported()
+{
+    return supported;
+}
+
+
 //---------------------------------------------------------------------------
 bool PolicyChecker::full_parse()
 {
-    // check for presence of _StringX or TimeCode* element in policy)
-    for (size_t pos=0; pos<rules.size(); pos++)
-    {
-        std::string& field=rules[pos]->field;
-        size_t index=field.find_last_not_of("0123456789");
-        if ((index!=std::string::npos && index>=6 && field.substr(index-6, 7)=="_String") || field.find("TimeCode")==0)
-            return true;
-    }
-
-    return false;
+    return full;
 }
 
 //---------------------------------------------------------------------------
@@ -326,6 +449,8 @@ PolicyChecker::RuleElement* PolicyChecker::parse_rule(tfsxml_string& tfsxml_priv
             rule->name=tfsxml_decode(attribute_value);
         else if (!tfsxml_strcmp_charp(attribute_name, "scope"))
             rule->scope=tfsxml_decode(attribute_value);
+        else if (!tfsxml_strcmp_charp(attribute_name, "level"))
+            rule->level=tfsxml_decode(attribute_value);
         else if (!tfsxml_strcmp_charp(attribute_name, "value"))
             rule->field=tfsxml_decode(attribute_value);
         else if (!tfsxml_strcmp_charp(attribute_name, "tracktype"))
@@ -336,18 +461,29 @@ PolicyChecker::RuleElement* PolicyChecker::parse_rule(tfsxml_string& tfsxml_priv
             rule->operand=tfsxml_decode(attribute_value);
     }
 
+    std::string occurrence=rule->occurrence;
+    if (occurrence.empty() || occurrence=="all" || occurrence=="any")
+        occurrence="*";
+
     tfsxml_string value;
     if (!tfsxml_value(&tfsxml_priv, &value))
         rule->requested=std::string(value.buf, value.len);
 
+    // Check for presence of _StringX or TimeCode* element in policy
+    size_t index=rule->field.find_last_not_of("0123456789");
+    if ((index!=std::string::npos && index>=6 && rule->field.substr(index-6, 7)=="_String") || rule->field.find("TimeCode")==0)
+        full=true;
+
+// Check for currently unsupported features
+    if (rule->scope=="mmt" || rule->requested=="compare")
+        supported=false;
+
     if (rule->scope.empty() || rule->scope=="mi")
     {
         std::stringstream ss;
-        ss << "mi:MediaInfo/mi:track[@type='" << rule->tracktype << "'][" << rule->occurrence << "]" << tokenize(rule->scope, rule->field, "/");
-        //if (rule->operand == "=" || rule->operand.rfind("<", 0) == 0 || rule->operand.rfind(">", 0) == 0 || rule->operand.rfind("&", 0) == 0 /* &lt; &lt;=...*/) // TODO: disable also for string fields
-        //    ss << rule->operand << "'" << rule->requested << "'";
+        ss << "mi:MediaInfo/mi:track[@type='" << rule->tracktype << "'][" << occurrence << "]" << tokenize(rule->scope, rule->field, "/");
         rule->xpath=ss.str();
-        rule->path = parse_path(rule->xpath);
+        rule->path=parse_path(rule->xpath);
     }
 
     rules.push_back(rule);
@@ -368,6 +504,8 @@ PolicyChecker::PolicyElement* PolicyChecker::parse_policy(tfsxml_string& tfsxml_
             policy->name=tfsxml_decode(attribute_value);
         else if (!tfsxml_strcmp_charp(attribute_name, "type"))
             policy->type=tfsxml_decode(attribute_value);
+        else if (!tfsxml_strcmp_charp(attribute_name, "level"))
+            policy->level=tfsxml_decode(attribute_value);
         else if (!tfsxml_strcmp_charp(attribute_name, "version"))
             policy->version=tfsxml_decode(attribute_value);
     }
@@ -427,7 +565,7 @@ void PolicyChecker::parse_node(tfsxml_string& tfsxml_priv, std::vector<RuleEleme
     tfsxml_string tfsxml_priv_save = tfsxml_priv;
     if (!tfsxml_enter(&tfsxml_priv))
     {
-        std::map<PathElement*, size_t> occurences;
+        std::map<PathElement*, size_t> occurrences;
 
         tfsxml_string result;
         while (!tfsxml_next(&tfsxml_priv, &result))
@@ -437,7 +575,7 @@ void PolicyChecker::parse_node(tfsxml_string& tfsxml_priv, std::vector<RuleEleme
             std::string field = std::string(result.buf, result.len);
             for (size_t pos = 0; pos < rules.size(); pos++)
             {
-                if (level < rules[pos]->path.size() && path_is_matching(tfsxml_priv, result, rules[pos]->path[level], occurences[&rules[pos]->path[level]]))
+                if (level < rules[pos]->path.size() && path_is_matching(tfsxml_priv, result, rules[pos]->path[level], occurrences[&rules[pos]->path[level]]))
                 {
                     tfsxml_string tfsxml_priv_copy = tfsxml_priv;
                     if (level == rules[pos]->path.size() - 1)
@@ -510,6 +648,7 @@ int PolicyChecker::analyze(const std::string& report, bool verbose, std::string&
 EMSCRIPTEN_BINDINGS(mediaconch_policychecker) {
     emscripten::class_<MediaConch::PolicyChecker>("PolicyChecker")
         .constructor()
+        .function("is_policy_supported", &MediaConch::PolicyChecker::is_policy_supported)
         .function("full_parse", &MediaConch::PolicyChecker::full_parse)
         .function("add_policy", &MediaConch::PolicyChecker::add_policy)
         .function("analyze", emscripten::optional_override([](MediaConch::PolicyChecker& self, const std::string& report, bool verbose) {
